/*
 * PM - 2016
 */

#define F_CPU 16000000

#include <stdint.h>
#include <avr/io.h>
#include <util/delay.h>

void init()
{
	PORTD |= (1 << PD6); //pull-up BTN
	DDRD &= ~(1 << PD6); //BTN
	DDRD |= (1 << PD7); //LED
	
	DDRB |= (1 << PB0); //motoare + si -
	DDRB |= (1 << PB1);
	DDRB |= (1 << PB2);
	DDRB |= (1 << PB3);
	DDRB |= (1 << PB4);
	DDRB |= (1 << PB5);
	DDRB |= (1 << PB6);
	DDRB |= (1 << PB7);
	
	DDRD |= (1 << PD0); //enable-uri motoare
	DDRD |= (1 << PD1);
	DDRD |= (1 << PD2);
	DDRD |= (1 << PD3);
	
	//adc
	ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0); //prescaler 128
	ADMUX = 0<<MUX0; //channel
	ADMUX |= (1 << REFS0); //reference voltage
	//ADCSRB = (0 << ADTS0); //free running mode (maybe just 1 input possible
	                       //  in freerunning mode)
	ADMUX |= (1 << ADLAR); //align left
	ADCSRA |= (1 << ADEN); //enable adc
	//ADCSRA |= (1 << ADSC); //start sampling
}

uint8_t ir_val;

inline void ir_refresh()
{
	ADCSRA |= (1 << ADSC); //start sampling
	while(ADCSRA & (1 << ADSC)); //wait for completion
	//ir_val = (uint8_t)((int16_t)256-(int16_t)ADCH);
	ir_val = ADCH;
}

//requires refresh
inline uint8_t ir()
{
	return ir_val;
}

//requires refresh
inline char ir_close() //close distance
{
	if (ir() < 64)
		return 0;
	return 1;
}

inline void led_on()
{
	PORTD |= (1 << PD7);
}

inline void led_off()
{
	PORTD &= ~(1 << PD7);
}

inline char btn()
{
	return (PIND & (1 << PD6)) == 0;
}

void motorApm(char p, char m)
{
	if (p == 1)
		PORTB |= (1 << PB0);
	else
		PORTB &= ~(1 << PB0);
	if (m == 1)
		PORTB |= (1 << PB1);
	else
		PORTB &= ~(1 << PB1);
}

void motorAen(char en)
{
	if (en == 1)
		PORTD |= (1 << PD0);
	else
		PORTD &= ~(1 << PD0);
}

void motorBpm(char p, char m)
{
	if (p == 1)
		PORTB |= (1 << PB2);
	else
		PORTB &= ~(1 << PB2);
	if (m == 1)
		PORTB |= (1 << PB3);
	else
		PORTB &= ~(1 << PB3);
}

void motorBen(char en)
{
	if (en == 1)
		PORTD |= (1 << PD1);
	else
		PORTD &= ~(1 << PD1);
}

void motorCpm(char p, char m)
{
	if (p == 1)
		PORTB |= (1 << PB4);
	else
		PORTB &= ~(1 << PB4);
	if (m == 1)
		PORTB |= (1 << PB5);
	else
		PORTB &= ~(1 << PB5);
}

void motorCen(char en)
{
	if (en == 1)
		PORTD |= (1 << PD2);
	else
		PORTD &= ~(1 << PD2);
}

void motorDpm(char p, char m)
{
	if (p == 1)
		PORTB |= (1 << PB6);
	else
		PORTB &= ~(1 << PB6);
	if (m == 1)
		PORTB |= (1 << PB7);
	else
		PORTB &= ~(1 << PB7);
}

void motorDen(char en)
{
	if (en == 1)
		PORTD |= (1 << PD3);
	else
		PORTD &= ~(1 << PD3);
}

#define N 160
#define D 25
char v[N];
int main(void) 
{
	init();
	
	motorAen(0);
	motorApm(0,0);
	
	motorBen(0);
	motorBpm(0,0);
	
	motorCen(0);
	motorCpm(0,0);
	
	motorDen(0);
	motorDpm(0,0);
	
	while (!btn())
	{
		ir_refresh();
		if (ir_close())
			led_on();
		else
			led_off();
		_delay_ms(64);
	}
		
	while (btn()) {}
	
	led_off();
	_delay_ms(1000);
	
	motorAen(1);
	motorBen(1);
	motorCen(1);
	motorDen(1);
	
	led_on();

	while(1)
	{
		while (!btn()) {}
		while (btn()) {}
		_delay_ms(1000);
		
		while(1)
		{
			if (btn())
			{
				while (btn()) {}
				break;
			}
			ir_refresh();
			if (!ir_close())
			{
				led_off();
				
				motorApm(1,0);	
				motorBpm(1,0);
				motorCpm(1,0);
				motorDpm(1,0);
				_delay_ms(64);
				
				while(1)
				{
					ir_refresh();
					if (ir_close())	
						break;
					_delay_ms(64);
				}
			}
			
			led_on();
			
			motorApm(0,1);	
			motorBpm(0,1);
			motorCpm(0,1);
			motorDpm(0,1);
			_delay_ms(256);
			
			motorApm(1,0);	
			motorBpm(0,1);
			motorCpm(1,0);
			motorDpm(0,1);
			_delay_ms(1024);
			
			while(1)
			{
				ir_refresh();
				if (!ir_close())
					break;
				_delay_ms(64);
			}
			
			led_off();
		}
	}

	return 0;
}
